@@ -30,11 +30,22 @@ class Token(dict):
 
         raise KeyError("'" + key + "'")
 
+    def __str__(self) -> str:
+        if 'form' in self:
+            return self['form']
+
+        if 'id' in self:
+            return f"id={self['id']}"
+
+        return ''
+
 class TokenList(T.List[Token]):
-
-    metadata: Metadata = Metadata()
-
-    def __init__(self, tokens: T.Iterable[Token] = None, metadata: Metadata = None):
+    def __init__(
+        self,
+        tokens: T.Iterable[Token] = None,
+        metadata: Metadata = None,
+        default_fields: T.Optional[T.Iterable[str]] = None,
+    ):
         tokens = tokens or []
 
         if not isinstance(tokens, list):
@@ -46,26 +57,28 @@ class TokenList(T.List[Token]):
         super(TokenList, self).__init__(tokens)
 
         self.metadata = metadata or Metadata()
+        self.default_fields = default_fields
 
     def __repr__(self) -> str:
-        return 'TokenList<' + ', '.join(token['form'] for token in self if 'form' in token) + '>'
+        return 'TokenList<' + ', '.join(str(token) for token in self) + '>'
 
     def __eq__(self, other: T.Any) -> bool:
         if not isinstance(other, TokenList):
             other = TokenList(other)
 
-        return self[:] == other[:] and self.metadata == other.metadata
+        return super(TokenList, self).__eq__(other) and self.metadata == other.metadata
 
     def __ne__(self, other: T.Any) -> bool:
         return not self == other
 
     def clear(self) -> None:
-        self[:] = []
+        super(TokenList, self).clear()
         self.metadata = Metadata()
+        self.default_fields = None
 
     def copy(self) -> 'TokenList':
         tokens_copy = super().copy()
-        return TokenList(tokens_copy, self.metadata)
+        return TokenList(tokens_copy, self.metadata, self.default_fields)
 
     def extend(self, iterable: T.Union['TokenList', T.Iterable[Token]]) -> None:
         if not isinstance(iterable, TokenList):
@@ -75,19 +88,39 @@ class TokenList(T.List[Token]):
 
         self.metadata.update(iterable.metadata)
 
-    def append(self, token: Token) -> None:
+    def _dict_to_token_and_set_defaults(self, token: T.Union[dict, Token]) -> Token:
         if not isinstance(token, Token):
             token = Token(token)
 
+        if self.default_fields:
+            for field in self.default_fields:
+                if field not in token:
+                    token[field] = "_"
+
+        return token
+
+    def append(self, token: T.Union[dict, Token]) -> None:
+        token = self._dict_to_token_and_set_defaults(token)
         super(TokenList, self).append(token)
 
-    @property
-    def tokens(self) -> 'TokenList':
-        return self
+    def insert(self, i: int, token: T.Union[dict, Token]) -> None:
+        token = self._dict_to_token_and_set_defaults(token)
+        super(TokenList, self).insert(i, token)
 
-    @tokens.setter
-    def tokens(self, value: T.Iterable[Token]) -> None:
-        self[:] = value
+    @T.overload
+    def __setitem__(self, key: int, tokens: T.Union[dict, Token]) -> None: ...  # noqa, pragma: no cover
+
+    @T.overload
+    def __setitem__(self, key: slice, tokens: T.Union[T.Iterable[T.Union[dict, Token]], 'TokenList']) -> None: ...  # noqa, pragma: no cover
+
+    def __setitem__(self, key, tokens):  # noqa: F811
+        if isinstance(key, int):
+            token = tokens
+            token = self._dict_to_token_and_set_defaults(token)
+            super(TokenList, self).__setitem__(key, token)
+        else:
+            tokens = [self._dict_to_token_and_set_defaults(token) for token in tokens]
+            super(TokenList, self).__setitem__(key, tokens)
 
     def serialize(self) -> str:
         return serialize(self)
@@ -140,7 +173,7 @@ class TokenList(T.List[Token]):
         return root
 
     def filter(self, **kwargs: T.Any) -> 'TokenList':
-        tokens: T.Iterable[Token] = self.tokens.copy()
+        tokens: T.Iterable[Token] = self.copy()
 
         for query, value in kwargs.items():
             filtered_tokens = []
